//////////////////////////////////////////////////////////////////////
//
// File					: conference.cpp
//
// Description			: Conference management functions
//
// Author				: Tomasz Stachowiak, NPAC
//
// Date					: 11-13-96
//
///////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NCSmanager.h"
#include "NCSmanagerDlg.h"
#include "AddUsers.h"
#include "ncs.h"
#include "conference.h"

// main dialog pointer
CNCSmanagerDlg *main_dlg;

// conference data
int conferenceExists = 0;
int iaminvited = 0;
Participant inviting_part;

//reading thread data
HANDLE hReadThread;
DWORD dwReadThreadId;

// plugins data
PLUGIN plugin[20];
int numPlugs;

// info about daemon process
PROCESS_INFORMATION daemon_info;

Participant activeParts[100];
int activePartsNum;

////////////////////////////////////////////////////////////////////////
//
// Conference management
//
////////////////////////////////////////////////////////////////////////
int start_conference(CNCSmanagerDlg *dlg)
{
	char path[100];
	char name[20] = "NCS_PATH";
	int len;
	main_dlg = dlg;
	 
	main_dlg->SetStatus("Initialization...please wait");

	len = GetEnvironmentVariable(name, path, 80);
	if(len == 0)
	{
		MessageBox(NULL, "Enviroment variable NCS_PATH not set", "Error", MB_ICONSTOP | MB_OK);
		exit(0);
	}

	strcat(path, DAEMON_PATH);
	startProcess(path, &daemon_info);

	Sleep(1500);

	//////////////////////////////////
	// NCS initialization            
	if(!ncsSessionInitialize())
	{
		MessageBox(NULL, "Session initialization error", "Error", MB_OK);
		return 0;
	}

	//////////////////////////////////
	// Callbacks setting
	ncsSessionAddCallback(indLocalSessionInvited, invited);
	ncsSessionAddCallback(indLocalSessionAddMember, new_member);
	ncsSessionAddCallback(indLocalSessionDeleteMember, left_member);
	ncsSessionAddCallback(indLocalSessionStartApp, start_app);
	ncsSessionAddCallback(indServerUsers, server_users);

	//////////////////////////////////
	// Reading thread creation
	hReadThread = CreateThread(
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) readingThread, // thread function 
        NULL,									// argument to thread function 
        0,										// use default creation flags 
        &dwReadThreadId);						// returns the thread identifier 
	if (hReadThread == NULL) 
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	
	read_plugins();

	
	ncsServerAdd();
	ncsServerUsers();
	
	main_dlg->SetStatus("Conferencing panel ready to use");

	return 1;
}

void exit_conference(void)
{
	main_dlg->SetStatus("Quitting...please wait");
	ncsServerDel();
	TerminateThread(hReadThread, 0);
	CloseHandle(hReadThread);
	ncsSessionQuit();
	quit_plugins();
	Sleep(2000);
	TerminateProcess(daemon_info.hProcess, 0);
}


void conf_reset(void)
{
	main_dlg->SetStatus("Conference reset...please wait");
	exit_conference();
	main_dlg->DeleteAllMembers();
	main_dlg->DeleteAllApps();
	conferenceExists = 0;
	Sleep(1000);
	start_conference(main_dlg);
}

void accept(void)
{
	if(iaminvited)
	{
		iaminvited = 0;
		if(!conferenceExists)
		{
			ncsSessionCreate();
			conferenceExists = 1;
		}
		// main_dlg->MessageBox(inviting_part.host);
		ncsSessionAcceptInvitation(&inviting_part);
	}
}

void reject(void)
{
	if(iaminvited)
	{
		iaminvited = 0;
		ncsSessionRejectInvitation(&inviting_part);
	}
}


////////////////////////////////////////////////////////////////////////
//
// Members management
//
////////////////////////////////////////////////////////////////////////
void parseIPaddress(char *IP, char *name, char *host)
{
  char *ptr, *pos;
  int len, i;

  ptr = IP;
  pos = strchr(ptr, 64); /* code of @ */
  len = (int)pos - (int)ptr;
  memcpy(name, ptr, len);
  name[len]='\0';
  ptr += len+1;
  memcpy(host, ptr, strlen(IP)-len-1);
  host[strlen(IP)-len-1]='\0';
} /* parseIPaddress() */

void add_member(char *user, char *host)
{
	Participant part;

	if(!conferenceExists)
	{
		ncsSessionCreate();
		conferenceExists = 1;
	}

	strcpy(part.user, user);
	strcpy(part.host, host);
	ncsSessionAddMember(&part);
}

void add_user(char *member)
{
	Participant part;

	if(!conferenceExists)
	{
		ncsSessionCreate();
		conferenceExists = 1;
	}
	parseIPaddress(member, part.user, part.host);		
	
	ncsSessionAddMember(&part);
}

void confAddMemberDlg()
{
	CAddUsers dlg;
	char text[100];

	ncsServerUsers();

	int nResponse = dlg.DoModal();

	if (nResponse == IDOK)
	{
	}	
	else if (nResponse == IDCANCEL)
	{
	}

}

////////////////////////////////////////////////////////////////////////
//
// Applications management
//
////////////////////////////////////////////////////////////////////////
void read_plugins(void)
{
	FILE *file;
	int i, len;
	char path[100];
	char name[20] = "NCS_PATH";
	char plugin_path[100];

	len = GetEnvironmentVariable(name, path, 80);
	strcat(path, PLUGINS_CFG_PATH);
	if((file = fopen(path, "r")) == NULL)
		MessageBox(NULL, "Can't read file", "Error", MB_OK);
	else
	{
		fscanf(file, "%d\n", &numPlugs);
		for(i=0; i<numPlugs; i++)
		{
			fscanf(file, "%d\n", &plugin[i].port);
			fgets(plugin[i].name, 50, file);
			plugin[i].name[strlen(plugin[i].name)-1] = 0;
			len = GetEnvironmentVariable(name, plugin[i].path, 80);
			fgets(plugin_path, 100, file);
			strcat(plugin[i].path, plugin_path);
			plugin[i].path[strlen(plugin[i].path)-1] = 0;
			plugin[i].started = 0;
			main_dlg->AddApp(plugin[i].name);
		}
	}
}


void quit_plugins(void)
{
	int i;
	
	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].started)
			TerminateProcess(plugin[i].info.hProcess, 0);
	}
}

void startProcess(char *path, PROCESS_INFORMATION *pinfo)
{
	STARTUPINFO sinfo;

	sinfo.cb = sizeof(STARTUPINFO);
	sinfo.lpReserved = NULL;
	sinfo.lpDesktop = NULL;
	sinfo.lpTitle = NULL;
	sinfo.dwFlags = STARTF_FORCEONFEEDBACK;
	sinfo.cbReserved2 = 0;
	sinfo.lpReserved2 = NULL;

	if(CreateProcess(path, NULL, NULL, NULL, FALSE, 
					NORMAL_PRIORITY_CLASS | DETACHED_PROCESS, NULL, NULL, 
					&sinfo, pinfo)
		== FALSE)
			MessageBox(NULL, "Create process error", "Error", MB_OK);
}


void create_app(int port)
{
	int found = 0, index, i;

	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].port == port)
		{
			index = i;
			found = 1;
		}
	}
	if((found) && (!plugin[index].started))
	{
		// MessageBox(NULL, plugin[index].path, "Info", MB_OK);
		plugin[index].started = 1;
		startProcess(plugin[index].path, &plugin[index].info);

	}
}

void start_local_app(int index)
{
	ncsSessionStartApp(plugin[index].port);
	create_app(plugin[index].port);
}

void quit_application(int port)
{
	int found = 0, index, i;

	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].port == port)
		{
			index = i;
			found = 1;
		}
	}
	if((found) && (plugin[index].started))
	{
		plugin[index].started = 0;
		TerminateProcess(plugin[index].info.hProcess, 0);
	}
}

void quit_local_app(int index)
{
	//ncsSessionSendToApp(plugin[index].port, indLocalAppQuit, NULL, 0);
	if(plugin[index].started)
	{
		plugin[index].started = 0;
		TerminateProcess(plugin[index].info.hProcess, 0);
	}
}

void info_app(int index)
{
	char text[200];

	wsprintf(text, "Port: %d\nName: %s\nPath: %s\n", plugin[index].port,
													 plugin[index].name,
													 plugin[index].path);
	main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONINFORMATION);
}

/////////////////////////////////////////////////////////////////////////
//
// NCS callbacks
//
/////////////////////////////////////////////////////////////////////////
void new_member(void *data)
{
	Participant *part;

	part = (Participant *)data;
	main_dlg->AddMember(part->user, part->host);
}

void left_member(void *data)
{
	Participant *part;

	part = (Participant *)data;
	main_dlg->DeleteMember(part->user, part->host);
}

void invited(void *data)
{
	char text[200];
	Participant *part;

	part = (Participant *)data;
	memcpy(&inviting_part, part, sizeof(Participant));
	iaminvited = 1;
	wsprintf(text, "You are invited for conference\nby %s@%s\nPress ACCEPT or REJECT\n", 
			 part->user, part->host);
	main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONEXCLAMATION);
}

void start_app(void *data)
{
	int *port;

	port = (int *)data;
	create_app(*port);
}

void server_users(void *data)
{
	InviteData *users;
	char text[100];
	int i;

	users = (InviteData *)data;
	activePartsNum = ntohl(users->numMembers);
	for(i=0; i<activePartsNum; i++)
	{
		memcpy(&activeParts[i], &users->parts[i], sizeof(Participant));
	}
}

/////////////////////////////////////////////////////////////////////////
//
// Reading thread
//
/////////////////////////////////////////////////////////////////////////
DWORD readingThread(LPDWORD data)
{
	struct timeval timeout;
	fd_set set;
	int sockfd;

	timeout.tv_sec = 0;
	timeout.tv_usec = 500000;
	while(1)
	{
		sockfd = ncsSessionGetFD(&set);
		if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
		{
			ncsProcessIncoming(&set);
		}
	}

	return 1;
}


////////////////////////////////  END  /////////////////////////////////////