/*********************************************************************
 *
 * File:         session_manage.c
 *
 * Description:  Session management module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/
#include <stdio.h>

#include "global.h"
#include "message_send.h"
#include "session_manage.h"

Session session;

int sendToApps(int app, short tag, char *data, int len)
{
  Message *msg;
  int size;
  char userid[20];
  char hostname[50];
  int i;
  DWORD user_len = 20;

  if(data != NULL)
    {
      size = sizeof(Message) + len - 1;
      msg = (Message *)malloc(size);
      if(msg == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      memcpy(msg->data, data, len);
    }
  else
    {
      size = sizeof(Message);
      msg = (Message *)malloc(size);
      if(msg == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      msg->data[0] = 0;
    }

  msg->tag = tag;
  GetUserName(userid, &user_len);
  getHostFullName(hostname, 50);
  strcpy(msg->sender.user, userid);
  strcpy(msg->sender.host, hostname);
  msg->local = LOCAL;

  if(app < 0)
    {
      for(i=0; i<MAX_APPS; i++)
	{
	  if(session.appActive[i])
	    {
	      PRINTF("Engine: send message to application\n");
	      if(sendControlPacket(&session.appAddr[i], (char *)msg, size) < 0)
		{
		  free(msg);
		  return 0; 
		}
	    }
	}
    }
  else
    {
      if(session.appActive[app])
	{
	  PRINTF("Engine: send message to application\n");
	  if(sendControlPacket(&session.appAddr[app], (char *)msg, size) < 0)
	    {
	      free(msg);
	      return 0; 
	    }
	
	}
    }
    
  free(msg);
  return 1;
}


int sendToAppPort(int port, short tag, char *data, int len)
{
  int i;

  for(i=0; i<MAX_APPS; i++)
    {
      if((session.appActive[i]) && (session.appPort[i] == port))
	sendToApps(i, tag, data, len);
    }
  return 1;
}


int newMember(Participant *member)
{
  MemberInfo info;
  struct hostent *hostinfo;

  hostinfo = gethostbyname(member->host);
  if(hostinfo == NULL)
    {
      fprintf(stderr, "Engine: new member unknown host %s\n", member->host);
      return 0;
    }

  session.lastMember++;
  session.numMembers++;
  session.memberActive[session.lastMember] = 1;
  strcpy(session.member[session.lastMember].user, member->user);
  strcpy(session.member[session.lastMember].host, member->host);

  session.memberAddr[session.lastMember].sin_family = AF_INET;
  session.memberAddr[session.lastMember].sin_addr.s_addr = 
    ((struct in_addr *)hostinfo->h_addr)->s_addr;
  session.memberAddr[session.lastMember].sin_port = htons(DAEMON_PORT);

   if(session.status != statusNonActive)
   {
		info.handle = session.lastMember;
		memcpy(&info.part, member, sizeof(Participant));
		memcpy(&info.addr, &session.memberAddr[session.lastMember], 
				sizeof(SOCKADDR_IN));
		sendControlMessage(LOCAL, 0, indLocalSessionAddMember, 
				(char *)&info, sizeof(MemberInfo));

		sendToApps(-1, indLocalSessionAddMember, (char *)&info, sizeof(MemberInfo));
   }

  return 1;
}

int getManagerAddr(SOCKADDR_IN *addr)
{
	char host_name[50];
	struct hostent *host_data;

	getHostFullName(host_name, 50);
	host_data = gethostbyname(host_name);

	addr->sin_family = AF_INET;
	addr->sin_addr.s_addr = *(long *)host_data->h_addr;
	addr->sin_port = htons(MANAGER_PORT);
	
	return 1;
}

int getUserData(int userNo, SOCKADDR_IN *addr, Participant *part)
{
  if((session.memberActive[userNo]) && (userNo != session.localMember))
    {
      memcpy(addr, &session.memberAddr[userNo], sizeof(SOCKADDR_IN));
      memcpy(part, &session.member[userNo], sizeof(Participant));
      return 1;
    }
  else
    return 0;
}

int getUserHandle(Participant *part)
{
  int i;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	if(   (strcmp(session.member[i].host, part->host) == 0)
	   && (strcmp(session.member[i].user, part->user) == 0))
	      return i;
    }

  return -1;
}



int sessionInit()
{
  int i;

  session.status = statusInit;
  session.lastMember = -1;
  session.numMembers = 0;
  session.localMember = -1;
  for(i=0; i<MAX_MEMBERS; i++)
    session.memberActive[i] = 0;

  for(i=0; i<MAX_APPS; i++)
    session.appActive[i] = 0;
  session.numApps = 0;
  session.lastApp = -1;

  return 0;
}


int sessionCreate(Participant *initiator)
{
  if(session.status == statusInit)
    {
      PRINTF("Engine: session initiator %s@%s\n", initiator->user, initiator->host);
      session.status = statusCreated;
      newMember(initiator);
      session.localMember = 0;
	  sessionSetLocalMember();
      return 1;
    }
  else
    {
      return 0;
    }
}

int sessionAddMember(Participant *newmember)
{
  int current;

  if((session.status == statusCreated) || (session.status == statusActive))
    {
      if(session.lastMember < MAX_MEMBERS-1)
	{
	  if(getUserHandle(newmember) == -1)
	    {
	      PRINTF("Engine: session member %s@%s\n", 
		     newmember->user, newmember->host);
	      session.status = statusActive;
	      newMember(newmember);
	      return 1;
	    }
	}
    }
  return 0;
}

void sessionDeleteMember(Participant *member)
{
  int i;

  for(i=0; i<MAX_MEMBERS; i++)
	{
	  if((session.memberActive[i]) 
	     && (!strcmp(member->user, session.member[i].user))
	     && (!strcmp(member->host, session.member[i].host)))
	    {
	      session.memberActive[i] = 0;
	      session.numMembers--;
	      sendControlMessage(LOCAL, 0, indLocalSessionDeleteMember, 
				 (char *)&i, sizeof(int));
	      sendToApps(-1, indLocalSessionDeleteMember, (char *)&i, sizeof(int));
	    }
	}
}


int sessionAddApp(SOCKADDR_IN *addr)
{
  MemberInfo info;
  int i;
  char host_name[50];
  struct hostent *host_data;

  if(session.lastApp < MAX_APPS - 1)
    {
      PRINTF("Engine: add application\n");
      session.lastApp++;
      session.numApps++;
      session.appActive[session.lastApp] = 1;
      session.appPort[session.lastApp] = ntohs(addr->sin_port);
      memcpy(&session.appAddr[session.lastApp], addr, 
	     sizeof(SOCKADDR_IN)); 
	  getHostFullName(host_name, 50);
	  host_data = gethostbyname(host_name);

      session.appAddr[session.lastApp].sin_addr.s_addr 
		  = *(long *)host_data->h_addr;

      for(i=0; i<MAX_MEMBERS; i++)
	{
	  if(session.memberActive[i])
	    {
	      info.handle = i;
	      memcpy(&info.part, &session.member[i], sizeof(Participant));
	      memcpy(&info.addr, &session.memberAddr[i], 
		     sizeof(SOCKADDR_IN));
	      sendToApps(session.lastApp, indLocalSessionAddMember,
			 (char *)&info, sizeof(MemberInfo));
	    }
	}

      sendToApps(session.lastApp, indLocalSessionLocalMember, 
		 (char *)&session.localMember, sizeof(int));

      return 1;
    }
  else
    return 0;
}


void sessionDeleteApp(int port)
{
  int i;

  for(i=0; i<MAX_APPS; i++)
    {
      if(session.appPort[i] == port)
	session.appActive[i] = 0;
    }
}

void sessionDesactivate()
{
  session.status = statusNonActive;
}

void sessionActivate()
{
  session.status = statusActive;
}

int sessionGetNoMembers()
{
  return session.numMembers;
}


void sessionGetMembers(Participant *parts)
{
  int i, no = 0;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	{
	  memcpy(parts, &session.member[i], sizeof(Participant));
	  parts++;
	}
    }
}

void sessionSetLocalMember()
{
  session.localMember = session.lastMember;
  sendToApps(-1, indLocalSessionLocalMember, 
	     (char *)&session.localMember, sizeof(int));
}

void sessionReject()
{
	if(session.numMembers == 1)
	{
		sessionDeleteMember(&session.member[session.localMember]);
		sessionInit();
	}
}









