/*********************************************************************
 *
 * File:         session_manage.h
 *
 * Description:  Header of session management module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/

#ifndef _SESSION_MANAGE_H
#define _SESSION_MANAGE_H

typedef enum _session_status
{
  statusNone,
  statusInit,
  statusCreated,
  statusNonActive,
  statusActive
} SessionStatus;

typedef struct _session
{
  SessionStatus status;

  int memberActive[MAX_MEMBERS];
  SOCKADDR_IN memberAddr[MAX_MEMBERS];
  Participant member[MAX_MEMBERS];
  int lastMember;
  int numMembers;
  int localMember;

  int appActive[MAX_APPS];
  int appPort[MAX_APPS];
  SOCKADDR_IN appAddr[MAX_APPS];
  int lastApp;
  int numApps;
} Session;


int sendToApps(int app, short tag, char *data, int len);
int sendToAppPort(int port, short tag, char *data, int len);

int newMember(Participant *member);
int getManagerAddr(SOCKADDR_IN *addr);
int getUserData(int userNo, SOCKADDR_IN *addr, Participant *part);

int sessionInit();
int sessionCreate(Participant *initiator);
int sessionAddMember(Participant *newmember);
void sessionDeleteMember(Participant *member);
int sessionAddApp(SOCKADDR_IN *addr);
void sessionDeleteApp(int port);
void sessionDesactivate();
void sessionActivate();
int sessionGetNoMembers();
void sessionGetMembers(Participant *parts);
void sessionSetLocalMember();
void sessionReject();

#endif
