///////////////////////////////////////////////////////////////////////////
// 
// File						: display.cpp
//
// Description				: Video display functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-21-96
//
////////////////////////////////////////////////////////////////////////////
#include <vfw.h>

#include "stdafx.h"
#include "DisplayDlg.h"
#include "h261.h"
#include "h263.h"
#include "convert.h"
#include "conference.h"
#include "display.h"
#include "yuvrgb24.h"

extern HANDLE hCodecEvent;

DisplayData dpyData[MAX_USERS];
BITMAPINFO *bmpInfo;

void displayInit(void)
{
	int i;
	
	bmpInfo = (BITMAPINFO *)GlobalAlloc(GPTR, sizeof(BITMAPINFO));

	// BITMAPINFO structure initialization (video format)
	//bmpInfo->bmiColors = (RGBQUAD *)NULL;
	bmpInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmpInfo->bmiHeader.biWidth = 176; //176;
	bmpInfo->bmiHeader.biHeight = 144; //144;
	bmpInfo->bmiHeader.biPlanes = 1;
	bmpInfo->bmiHeader.biBitCount = 24;
	bmpInfo->bmiHeader.biCompression = BI_RGB;
	bmpInfo->bmiHeader.biSizeImage = 0;
	bmpInfo->bmiHeader.biXPelsPerMeter = 0;
	bmpInfo->bmiHeader.biYPelsPerMeter = 0;	
	bmpInfo->bmiHeader.biClrUsed = 0;
	bmpInfo->bmiHeader.biClrImportant = 0;	

	for(i=0; i<MAX_USERS; i++)
	{
		dpyData[i].first = 1;
		dpyData[i].wndActive = 0;
	}

	init_dither_tab();
}

void displayOpen(DWORD user)
{
	char text[100];

	//wsprintf(text, "User %d", user);
	//MessageBox(NULL, text, "Info", MB_OK);

	// new display dialog
	dpyData[user].dpy = new DisplayDlg;

	if(!dpyData[user].dpy->Create(IDD_DISPLAY))
		MessageBox(NULL, "Error creating window", "Error", MB_OK);

	dpyData[user].dpy->SetWindowPos(&dpyData[user].dpy->wndTopMost, 200*user+100, 100, 0, 0, SWP_NOSIZE);

	dpyData[user].cDc = new CClientDC(dpyData[user].dpy);
	dpyData[user].cDc->SetMapMode(MM_TEXT);
	dpyData[user].hDc = dpyData[user].cDc->GetSafeHdc();

	dpyData[user].hDraw = DrawDibOpen();

	if(!DrawDibBegin(dpyData[user].hDraw, dpyData[user].hDc, 
						176, 144, 
						&bmpInfo->bmiHeader, -1, -1, DDF_HALFTONE ))
		MessageBox(NULL, "Draw Dib begin error", "Error", MB_OK);

	dpyData[user].wndActive = 1;
  }

void displayH261(void *data, int len, DWORD user)
{
	unsigned char *yuv;
	char *image;
	char text[100];

	//wsprintf(text, "Data size %d, user %d", len, user);
	//MessageBox(NULL, text, "Info", MB_OK);

	if(dpyData[user].wndActive)
	{
		yuv = (unsigned char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3/2);
		image = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3);

		WaitForSingleObject(hCodecEvent, INFINITE);
		ResetEvent(hCodecEvent);
		if(dpyData[user].first)
		{
			p64DecodeFrame((char *)data, len, (char *)yuv, 1, user);
			dpyData[user].first = 0;
		}
		else
			p64DecodeFrame((char *)data, len, (char *)yuv, 0, user);
		SetEvent(hCodecEvent);

	
		ConvertYUVtoRGB(yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
						(unsigned char *)image,
						176, 144);
	
		DrawDibDraw(dpyData[user].hDraw, dpyData[user].hDc, 0, 0, 176, 144, 
			&bmpInfo->bmiHeader, image, 
			0, 0, 176, 144, DDF_HALFTONE);
	
		GlobalFree(yuv);
		GlobalFree(image);
	}
}


void displayH263(void *data, int len, DWORD user)
{
	char *image;
	char text[100];
	unsigned char **pYUV;

	//wsprintf(text, "Data size %d, user %d", len, user);
	//MessageBox(NULL, text, "Info", MB_OK);

	if(dpyData[user].wndActive)
	{
		image = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3);

		pYUV = H263decodeFrame((char *)data, len, user);
	
		ConvertYUVtoRGB(pYUV[0], pYUV[1], pYUV[2],
						(unsigned char *)image,
						176, 144);
	
		DrawDibDraw(dpyData[user].hDraw, dpyData[user].hDc, 0, 0, 176, 144, 
			&bmpInfo->bmiHeader, image, 
			0, 0, 176, 144, DDF_HALFTONE);
	
		GlobalFree(image);
	}
}

void displayClose(DWORD user)
{
	delete dpyData[user].cDc;
	dpyData[user].dpy->DestroyWindow();
	delete dpyData[user].dpy;
	DrawDibClose(dpyData[user].hDraw);
}

void displayQuit(void)
{
	int i;

	for(i=0; i<MAX_USERS; i++)
	{
		if(dpyData[i].wndActive)
			displayClose(i);
	}
	GlobalFree(bmpInfo);
}