///////////////////////////////////////////////////////////////////////////
// 
// File						: conference.cpp
//
// Description				: conference functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-21-96
//
////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NCSvideo.h"
#include "NCSvideoDlg.h"
#include "PanelDlg.h"
#include "display.h"
#include "conference.h"
#include "ncs.h"
#include "VideoProtocol.h"
#include "h261.h"
#include "h263.h"
#include "capture.h"


//////////////////////////////////////////////////
// Conference global variables
//
CNCSvideoDlg *main_dlg;
ncsApplication *ncsApp;
DWORD localH;
HANDLE hThread;
DWORD dwThreadId;
int state;
int code;
int intra;
BOOL H261IntraMode;
DWORD codeTime=500;
DWORD decodeTime;
HANDLE hCodecEvent;
HANDLE hCtrlThread;
DWORD dwCtrlThreadId;
BOOL test;

/////////////////////////////////////////////////////////////////////////////////////
//
// Initialization
//
void confInit(CNCSvideoDlg *dlg)
{
	DWORD wParam;
	
	main_dlg = dlg;
	
	// Start up NCS
	ncsApp = (ncsApplication *)ncsAppAttach(VIDEO_PLUGIN_PORT);

	ncsAppAddCallback(ncsApp, tagAppQuit, (void (*)())confClose);

	//localH = ncsAppGetLocalHandle(ncsApp);
	
	// Create reading thread
	hThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) confReadThread,	// thread function 
			NULL,									// argument to thread function 
			0,										// use default creation flags 
			&dwThreadId);							// returns the thread identifier 
	
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}

	hCtrlThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) confTimeCtrlThread,	// thread function 
			NULL,									// argument to thread function 
			0,										// use default creation flags 
			&dwCtrlThreadId);							// returns the thread identifier 
	
	if (hCtrlThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}
	
	p64InitCodec();
	p64InitEncoder();
	p64InitDecoder();

	H263initEncoder(176, 144);
	H263initDecoder();

	displayInit();

	hCodecEvent = CreateEvent(NULL, TRUE, TRUE, "codec_event");

	state = STOP;
	code = H263;
	intra = 1;
	H261IntraMode = FALSE;
	test = TRUE;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Main reading thread
//
DWORD confReadThread(LPDWORD lpdwParam)
{
	struct timeval timeout;
	fd_set set;
	ncsData *ncsdata;
	int size;
	int sockfd;
	char *data;

	timeout.tv_sec = 0;
	timeout.tv_usec = 100000;

	while(1)
	{
		sockfd = ncsAppGetFD(ncsApp, &set);
		if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
		{
			ncsAppProcessIncoming(ncsApp, &set);
			size = ncsAppGetData(ncsApp, &set, &data);
			if(size > 0)
			{
				size = size - sizeof(ncsData) + 1;
				ncsdata = (ncsData *)data;
				ncsdata->handle = ntohl(ncsdata->handle);
				ncsdata->type = ntohl(ncsdata->type);
				confHandleData(ncsdata->type, ncsdata->data, size, 
								       ncsdata->handle);
				SetFocus(main_dlg->m_hWnd);
				free(data);
			}
		}
	}
	return 1;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Conference data handling procedure
//
void confHandleData(DWORD tag, void *data, int len, DWORD handle)
{
	char buffer[1600];
	VideoPluginFrame *frame;
	DWORD wParam;
	int size;
	
	// MessageBox(NULL, "Get audio data", "Info", MB_OK);
	switch (tag)
	{
	case H261:
		frame = (VideoPluginFrame *)data;
		size = len - sizeof(VideoPluginFrame) + 1;
		displayH261(frame->data, size, handle);
		break;
    
	case H263:
		frame = (VideoPluginFrame *)data;
		size = len - sizeof(VideoPluginFrame) + 1;
		displayH263(frame->data, size, handle);
		break;
	
	case videoTagExit:
        // exit plugin 
		break;

    case videoTagConfirm:
		break;

    case videoTagPlay:
		//displayOpen(handle);
		wParam = CREATE_DISPLAY*65536 + handle;
		SendMessage(main_dlg->m_hWnd, WM_COMMAND, wParam, NULL); 
		confSendMessage(videoTagConfirm, NULL, 0);
		break;

    case videoTagStop:
        break;

    case videoTagFrameDecoded:
		break;

    default:
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// 
void confSendMessage(DWORD tag, void *data, int len)
{
	ncsAppSendData(ncsApp, -1, tag, data, len);
}


void confSendVideo(int tag, char *buf, int len)
{
	VideoPluginFrame *frame;
	int size;

	size = sizeof(VideoPluginFrame) + len - 1;
	frame = (VideoPluginFrame *)GlobalAlloc(GPTR, size);
	//gettimeofday(&frame->time);
	memcpy(frame->data, buf, len);
	confSendMessage(tag, frame, size);
	GlobalFree(frame);
}

void confCodeImage(char *yuv)
{
	char *codedBuf;
	int size;
	DWORD start, stop;

	start = GetTickCount();
	
	codedBuf = (char *)GlobalAlloc(GPTR, 20000);

	switch(code)
	{
	case H261:
		WaitForSingleObject(hCodecEvent, INFINITE);
		ResetEvent(hCodecEvent);
	
		size = p64EncodeFrame(yuv, codedBuf);
	
		SetEvent(hCodecEvent);
	
		confSendVideo(H261, codedBuf, size);

		break;
	
	case H263:
		if(intra)
		{
			size = H263codeIntraFrame((unsigned char *)yuv, codedBuf);
			intra = 0;
		}
		else
			size = H263codeFrame((unsigned char *)yuv, codedBuf);

		confSendVideo(H263, codedBuf, size);
	
		break;
	}
	
	GlobalFree(codedBuf);
	
	stop = GetTickCount();
	codeTime = stop - start;
}

void confPlay(void)
{
	if(state == STOP)
	{
		state = PLAY;
		confSendMessage(videoTagPlay, NULL, 0);
		captureInit(main_dlg->m_hWnd, 500);
		captureGrab();
		test = TRUE;
	}
}

void confStop(void)
{
	if(state == PLAY)
	{
		state = STOP;
		captureClose();
		confSendMessage(videoTagStop, NULL, 0);
		intra = 1;
	}
}

void confPanel(void)
{
	CPanelDlg *panel;
	BOOL restart=FALSE;
	int nResponse;

	if(state == PLAY)
	{
		confStop();
		restart = TRUE;
	}

	if(code == H261)
	{
		panel = new CPanelDlg(TRUE, H261IntraMode);
	}
	else
	{
		panel = new CPanelDlg(FALSE, H261IntraMode);
	}

	nResponse = panel->DoModal();
	if (nResponse == IDOK)
	{
		if(panel->Code261)
			code = H261;
		else
			code = H263;

		if(panel->m_IntraOnly)
		{
			p64setIntraMode();
			H261IntraMode = TRUE;
		}
		else
		{
			p64resetIntraMode();
			H261IntraMode = FALSE;
		}
	}
	else if (nResponse == IDCANCEL)
	{
	}

	delete panel;

	if(restart)
		confPlay();
}

void confClose(void)
{
	CloseHandle(hCodecEvent);
	displayQuit();
	if(state == PLAY)
		captureClose();
	TerminateThread(hThread, 0);
	CloseHandle(hThread);
	ncsAppDettach(ncsApp);
}

DWORD confTimeCtrlThread(LPDWORD lpdwParam)
{
	DWORD wParam;

	while(1)
	{
		Sleep(10000);
		if((state == PLAY) && (test))
		{
			wParam = UPDATE_RATE*65536;
			//SendMessage(main_dlg->m_hWnd, WM_COMMAND, wParam, NULL); 
		}
	}
	return 0;
}

void confUpdateRate()
{
	if(code == H263)
		captureSetRate((double)codeTime * (double)3);
	else
		if(!H261IntraMode)
			captureSetRate((double)codeTime * (double)4);
		else
			captureSetRate((double)codeTime * (double)5);

	test = FALSE;
}
