//////////////////////////////////////////////////////////////////////////
//
// File					: convert.cpp
//
// Description			: Convertion functions between RGB 16 and YUV 411
//
///////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "convert.h"

#define COEFFR 1.1
#define COEFFG 1.1
#define COEFFB 1.1

unsigned char getB(WORD rgb)
{
	unsigned char result;

	result = ((unsigned char)(rgb & 0x1F)) << 3;
	return (float)COEFFB*(float)result;
}

unsigned char getG(WORD rgb)
{
	unsigned char result;

	result = ((unsigned char)((rgb >> 5) & 0x1F)) << 3;
	return (float)COEFFG*(float)result;
}

unsigned char getR(WORD rgb)
{
	unsigned char result;

	result = ((unsigned char)((rgb >> 10) & 0x1F)) << 3;
	return (float)COEFFR*(float)result;
}

void putB(WORD *rgb, char r)
{
	WORD pom;

	pom = (r>>3) & 0x001F;
	*rgb = *rgb | pom;
}

void putG(WORD *rgb, char r)
{
	WORD pom;

	pom = (r << 2) & 0x03E0;
	*rgb = *rgb | pom;
}

void putR(WORD *rgb, char r)
{
	WORD pom;

	pom = (r << 7) & 0xEC00;
	*rgb = *rgb | pom;
}



void rgb2yuv(WORD *input, 
			 unsigned char *y, unsigned char *u, unsigned char *v, 
			 int width, int height)
{
	int i, j;
	short cru[4], crv[4], lum;
	int QCIFwidth;

	QCIFwidth = 176;
	for(i=0; i<height; i++)
	{
		for(j=0; j<width; j++)
		{
			lum =	.299*(unsigned float)getR(input[i*width + j]) 
				+	.587*(unsigned float)getG(input[i*width + j])
				+	.114*(unsigned float)getB(input[i*width + j]);
			
			lum = (float)((float)219/(float)255)*(unsigned float)lum + 16;
			y[(height-i)*QCIFwidth + (j)] = lum;

			if(((i%2) == 1) && ((j%2) == 1))
			{
				cru[0] =	(-0.1687)*(unsigned float)getR(input[i*width + j])
					+		(-0.3313)*(unsigned float)getG(input[i*width + j])
					+		(0.5)	 *(unsigned float)getB(input[i*width + j]);
				
				cru[1] =	(-0.1687)*(unsigned float)getR(input[i*width + j - 1])
					+		(-0.3313)*(unsigned float)getG(input[i*width + j - 1])
					+		(0.5)	 *(unsigned float)getB(input[i*width + j - 1]);
				
				cru[2] =	(-0.1687)*(unsigned float)getR(input[i*(width-1) + j])
					+		(-0.3313)*(unsigned float)getG(input[i*(width-1) + j])
					+		(0.5)	 *(unsigned float)getB(input[i*(width-1) + j]);
				
				cru[3] =	(-0.1687)*(unsigned float)getR(input[i*(width-1) + j - 1])
					+		(-0.3313)*(unsigned float)getG(input[i*(width-1) + j - 1])
					+		(0.5)	 *(unsigned float)getB(input[i*(width-1) + j - 1]);

				u[(((height-i)/2)*(QCIFwidth/2)) + ((j)/2)] = (unsigned char)((((float)224/(float)255)*(unsigned float)((cru[0] + cru[1] + cru[2] + cru[3])/4)) + 128);

				crv[0] =	(0.5)    *(unsigned float)getR(input[i*width + j])
					+		(-0.4187)*(unsigned float)getG(input[i*width + j])
					+		(-0.0813)*(unsigned float)getB(input[i*width + j]);
				
				crv[1] =	(0.5)    *(unsigned float)getR(input[i*width + j - 1])
					+		(-0.4187)*(unsigned float)getG(input[i*width + j - 1])
					+		(-0.0813)*(unsigned float)getB(input[i*width + j - 1]);
				
				crv[2] =	(0.5)    *(unsigned float)getR(input[i*(width-1) + j])
					+		(-0.4187)*(unsigned float)getG(input[i*(width-1) + j])
					+		(-0.0813)*(unsigned float)getB(input[i*(width-1) + j]);
				
				crv[3] =	(0.5)    *(unsigned float)getR(input[i*(width-1) + j - 1])
					+		(-0.4187)*(unsigned float)getG(input[i*(width-1) + j - 1])
					+		(-0.0813)*(unsigned float)getB(input[i*(width-1) + j - 1]);

				v[(((height-i)/2)*(QCIFwidth/2)) + ((j)/2)] = (unsigned char)((((float)224/(float)255)*(unsigned float)((crv[0] + crv[1] + crv[2] + crv[3])/4)) + 128);
			}
		}
	}
}


void yuv2rgb(char *y, char *u, char *v, char *out, int width, int height)
{
	int i, j;
	char r, g, b;

	for(i=0; i<height; i++)
	{
		for(j=0; j<width; j++)
		{
			r = u[(i*width + j)/4] + y[i*width + j];
			g = y[i*width + j] - 0.194 * (float)u[(i*width + j)/4] - 0.509 * (float)v[(i*width + j)/4];
			b = v[(i*width + j)/4] + y[i*width + j];
			out[3*(i*width + j) + 0] = b;
			out[3*(i*width + j) + 1] = g;
			out[3*(i*width + j) + 2] = r;
			
			
			
			
			/*
			// for 16bit RGB conversion
			putR(&out[(i*width + j)], r);
			putG(&out[(i*width + j)], g);
			putB(&out[(i*width + j)], b);
			*/
		}
	}
}