/*********************************************************************
 *
 * File:         common.h
 *
 * Description:  Common definitions for session and application 
 *               management API
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-16-96
 *
 *********************************************************************/
#ifndef _NCS_H
#define _NCS_H

#include <stdio.h>
#include <winsock.h>

// #define QUIET

#ifndef QUIET
#define PRINTF printf
#else
#define PRINTF nic 
#endif


#define MAX_UDP_SIZE 60000

#define MAX_MEMBERS 20

#define DAEMON_PORT 5000
#define MANAGER_PORT 4999


/*********************************************************************
 *
 * NCS Data types
 *
 *********************************************************************/
typedef struct _participant
{
  char user[20];
  char host[50];
} Participant;

typedef struct _message
{
  unsigned char local;
  unsigned short tag;
  Participant sender;
  Participant receiver;
  char data[1];
} Message;

/* Tags in local messages */
typedef enum _local_tag
{
  /* local requests */
  reqLocalSessionCreate,
  reqLocalSessionQuit,
  reqLocalSessionInit,
  reqLocalSessionAddMember,
  reqLocalSessionJoin,
  reqLocalSessionReject,
  reqLocalSessionGetMembers,
  reqLocalSessionSendToApp,
  reqLocalSessionStartApp,

  reqLocalApplicationAttach,
  reqLocalApplicationDettach,

  /* local indications */
  indLocalSessionInvited,
  indLocalSessionAccept,
  indLocalSessionReject,
  indLocalSessionMembers,
  indLocalSessionAddMember,
  indLocalSessionDeleteMember,
  indLocalSessionLocalMember,
  indLocalSessionStartApp,

  indLocalAppQuit
} LocalTag;

typedef struct _manager_data
{
  int socket;
  struct sockaddr_in addr;
  struct sockaddr_in daemon_addr;
} ManagerData;

typedef struct _invite_data
{
  int numMembers;
  Participant parts[1];
} InviteData;

typedef struct _member_info
{
  int handle;
  Participant part;
  struct sockaddr_in addr;
} MemberInfo;

typedef struct _ncs_application
{
  int port;
  int socket;
  int ctrl_socket;
  struct sockaddr_in addr;
  struct sockaddr_in ctrl_addr;
  struct sockaddr_in daemon_addr;
  MemberInfo member[MAX_MEMBERS];
  int memberActive[MAX_MEMBERS];
  int numMembers;
  int localHandle;
  void (* startingCallback)(void);
  void (* quitCallback)(void);
} ncsApplication;


typedef struct _ncs_data
{
  DWORD type;
  DWORD handle;
  char data[1];
} ncsData;

typedef enum _app_tag
{
  tagAppStarting,
  tagAppQuit
} AppTag;

typedef struct _app_data
{
  int port;
  short tag;
  char data[1];
} AppData;


//
// common.cpp functions declarations
/////////////////////////////////////////////////////////////////////////
int sendControlPacket(struct sockaddr_in *addr, char *msg, int len);
int ncsEngineRequest(short tag, char *data, int len);
int sendPacket(int socket, struct sockaddr_in *addr, char *msg, int len);


/************************************************************************
 * 
 * NCS API functions declarations
 *
 ***********************************************************************/

/* Session managment functions */

int ncsSessionInitialize();
void ncsSessionCreate();
int  ncsSessionGetFD(fd_set *set);
void ncsProcessIncoming(fd_set *set);
void ncsSessionAddMember(Participant *part);
void ncsSessionAcceptInvitation(Participant *part);
void ncsSessionRejectInvitation(Participant *part);
void ncsSessionAddCallback(short tag, void (*function));
void ncsSessionQuit();
void ncsSessionQueryMembers();
void ncsSessionSendToApp(int port, short type, char *msg, int len);
void ncsSessionStartApp(int port);


/* Applications management functions */

ncsApplication *ncsAppAttach(int port);
void ncsAppDettach(ncsApplication *app);
int ncsAppSendData(ncsApplication *app, int mbr, DWORD type, 
		   void *data, int len);
int ncsAppProcessIncoming(ncsApplication *app, fd_set *set);
int ncsAppGetData(ncsApplication *app, fd_set *set, char **data);
int ncsAppGetFD(ncsApplication *app, fd_set *set);
DWORD ncsAppGetLocalHandle(ncsApplication *app);
int ncsAppGetNumMembers(ncsApplication *app);
void ncsAppAddCallback(ncsApplication *app, short tag, void (* function));


#endif






