/*********************************************************************
 *
 * File:         daemon.c
 *
 * Description:  Daemon for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <winsock.h>

#include "global.h"
#include "session_manage.h"
#include "daemon.h"

char localHost[50];   /* local host name */

/* System sockets */
int serverSocket;          /* main listening socket */
int tmpSocket;             /* temporary socket */
int inviteSocket;             /* invitation receive and handle socket */
int localSocket;           /* socket to communicate with CM */
int mbrSocket[MAX_MEMBERS];  /* member sockets */
int appSocket[MAX_APPS];   /* applications sockets */

int dbState;

struct sockaddr_in tmpAddr;

struct sockaddr_in addr;   /* address to bind to */

void nic()
{
}


int readAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = recv( fd, (char *)buffer, len, 0 );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "readAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}

int readMsgHeader(int sock, MsgHeader *header)
{
  char buf[MSG_HEADER_LENGTH];
  int err;

  PRINTF("Engine: reading header\n");
  err = readAll(sock, buf, MSG_HEADER_LENGTH);
  if(err == 0)
    {
      fprintf(stderr, "Engine: connection broken\n");
      return err;
    }
  if(err < 0)
    {
      fprintf(stderr, "Engine: reading error\n");
      return err;
    }

  header->tag = buf[0];
  header->session = buf[1];
  header->sender.handle = buf[2];
  strcpy(header->sender.user, &buf[3]);
  header->sender.host = *(int *)&buf[23];
  //header->sender.host = ntohl(*(int *)&buf[23]);
  header->receiver.handle = buf[27];
  strcpy(header->receiver.user, &buf[28]);
  header->receiver.host = *(int *)&buf[48];
  //header->receiver.host = ntohl(*(int *)&buf[48]);
  header->dataLength = ntohl(*(int *)&buf[52]);

  return err;
}

int getFreeMbrSocket()
{
  int found=0;
  int i=0;

  while((i < MAX_MEMBERS) && (!found))
    {
      if((mbrSocket[i] < 0) && (i != getLocalHandle()))
	found = 1;
      else
	i++;
    }
  if(found)
    return i;
  else
    return -1;
}

int getFreeAppSocket()
{
  int found=0;
  int i=0;

  while((i < MAX_APPS) && (!found))
    {
      if(appSocket[i] < 0)
	found = 1;
      else
	i++;
    }
  if(found)
    return i;
  else
    return -1;
}

void setFD(fd_set *set)
{
  int i;

  FD_ZERO(set);

  for(i=0; i<MAX_MEMBERS; i++)
    if(mbrSocket[i] >= 0)
      FD_SET(mbrSocket[i], set);

  for(i=0; i<MAX_APPS; i++)
    if(appSocket[i] >= 0)
      FD_SET(appSocket[i], set);

  if(localSocket >= 0)
    FD_SET(localSocket, set);

  if(inviteSocket >= 0)
    FD_SET(inviteSocket, set);
    
  if(tmpSocket >= 0)
    FD_SET(tmpSocket, set);

  FD_SET(serverSocket, set);
}


int openServerSocket()
{
  if ((serverSocket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    fprintf(stderr, "Engine: can't open daemon socket\n");

  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = htonl(INADDR_ANY);
  addr.sin_port        = htons(DAEMON_PORT);

  if(bind(serverSocket, (struct sockaddr *)&addr, sizeof(addr)) < 0)
    {
      fprintf(stderr, "Engine: can't bind daemon socket\n");
      quitDaemon();
      closesocket(serverSocket);
      exit(0);
    }

  listen(serverSocket, 5);

  return 1;
}

/* Daemon initialization */
void initDaemon()
{
  int i;
  WSADATA wsaData;	

	printf("Daemon init\n");

	if(WSAStartup(257, &wsaData) != 0)
	  fprintf(stderr, "Engine: can't startup winsock\n");

  for(i=0; i<MAX_MEMBERS; i++)
    mbrSocket[i] = -1;

  for(i=0; i<MAX_APPS; i++)
    appSocket[i] = -1;

  tmpSocket = -1;
  inviteSocket = -1;
  localSocket = -1;

  openServerSocket();

  //getHostFullName(localHost, 50);

  sessionAbandon();

  dbState = DbStateIdle;

	PRINTF("Daemon initialized\n");
}


/* Daemon quitting */
void quitDaemon()
{
  int i;

  sessionAbandon();

  for(i=0; i<MAX_MEMBERS; i++)
    if(mbrSocket[i] >= 0)
      {
	closesocket(mbrSocket[i]);
	mbrSocket[i] = -1;
      }

  for(i=0; i<MAX_APPS; i++)
    if(appSocket[i] >= 0)
      {
	closesocket(appSocket[i]);
	appSocket[i] = -1;
      }

  if(localSocket >= 0)
    {
      closesocket(localSocket);
      localSocket = -1;      
    }

  if(inviteSocket >= 0)
    {
      closesocket(inviteSocket);
      inviteSocket = -1;
    }

  if(tmpSocket >= 0)
    {
      closesocket(tmpSocket);
      tmpSocket = -1;
    }
}

/**************************************************************************
 *
 * Main socket reading procedure
 *
 **************************************************************************/
void runDaemon()
{
  fd_set set;
  struct timeval timeout;
  int len;
  int i;

  len = sizeof(struct sockaddr_in);
  timeout.tv_sec = 1;
  timeout.tv_usec = 0;
  while(1)
    {
      setFD(&set);
      if(select(FD_SETSIZE, &set, 0, 0, &timeout) > 0)
	{
	  for(i=0; i<MAX_MEMBERS; i++)
	      if((mbrSocket[i] >= 0) && (FD_ISSET(mbrSocket[i], &set)))
		readMemberMessage(i);
	  
	  for(i=0; i<MAX_APPS; i++)
	    if((appSocket[i] >= 0) && (FD_ISSET(appSocket[i], &set)))
	      readAppMessage(i);
	  
	  if((localSocket >= 0) && (FD_ISSET(localSocket, &set)))
	    readLocalMessage();
	  
	  if((tmpSocket >= 0) && (FD_ISSET(tmpSocket, &set)))
	    readNewMessage();

	  if(FD_ISSET(serverSocket, &set))
	    {
	      if(tmpSocket < 0)
		{
		  tmpSocket = accept(serverSocket, (struct sockaddr *)&tmpAddr, &len);	    
		  if(tmpSocket < 0)
		    fprintf(stderr, "Engine: accept error\n");
		  else
		    PRINTF("Accepted\n");
		}
	    }
	}
    }
}

/*********************************************************************
 *
 * Incoming messages reading procedures
 *
 *********************************************************************/
void readMemberMessage(int no)
{
  char *msg;
  MsgHeader header;
  int err;

  PRINTF("Engine: read member message\n");
  err = readMsgHeader(mbrSocket[no], &header);
  if(err == 0)
    {
      PRINTF("Engine: member left %d\n", no);
      sessionDeleteMember(no);
      closesocket(mbrSocket[no]);
      mbrSocket[no] = -1;
      /* member left handle */
      return;
    }

  if(err < 0)
    {
      fprintf(stderr, "Engine: data reading error\n");
      return;
    }

  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(mbrSocket[no], msg, header.dataLength);
      if(err == 0)
	{
	  fprintf(stderr, "Engine: member left\n");
	  sessionDeleteMember(no);
	  closesocket(mbrSocket[no]);
	  mbrSocket[no] = -1;
	  /* member left handle */
	  return;
	}
      
      if(err < 0)
	{
	  fprintf(stderr, "Engine: data reading error\n");
	  return;
	}
    }
  else
    msg = NULL;
  
  handleMemberMessage(no, &header, msg, header.dataLength);
  
  free(msg);
}

void readAppMessage(int no)
{
  char *msg;
  MsgHeader header;
  int err;

  PRINTF("Engine: read application message\n");

  err = readMsgHeader(appSocket[no], &header);
  if(err == 0)
    {
      /* application dettached */
      sessionDeleteApp(no);
      closesocket(appSocket[no]);
      appSocket[no] = -1;
      return;
    }
  if(err < 0)
    {
      fprintf(stderr, "Engine: data reading error\n");
      return;
    }

  PRINTF("App message length %d\n", header.dataLength);
  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(appSocket[no], msg, header.dataLength);
      if(err == 0)
	{
	  /* application dettached */
	  sessionDeleteApp(no);
	  closesocket(appSocket[no]);
	  appSocket[no] = -1;
	  return;
	}
      if(err < 0)
	{
	  fprintf(stderr, "Engine: data reading error\n");
	  return;
	}
    }
  else
    msg = NULL;
  
  handleAppMessage(no, &header, msg, header.dataLength);
  
  free(msg);
}

void readLocalMessage()
{
  char *msg;
  MsgHeader header;
  int err;

  PRINTF("Engine: read local message\n");

  err = readMsgHeader(localSocket, &header);
  if(err == 0)
    {
      PRINTF("Engine: local connection broken -- exiting...\n");
      quitDaemon();
    }
  if(err < 0)
    {
      fprintf(stderr, "Engine: data reading error\n");
      return;
    }

  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(localSocket, msg, header.dataLength);
      if(err == 0)
	{
	  PRINTF("Engine: local connection broken -- exiting...\n");
	  quitDaemon();
	}
      if(err < 0)
	{
	  fprintf(stderr, "Engine: data reading error\n");
	  return;
	}
    }
  else
    msg = NULL;
  
  handleLocalMessage(&header, msg, header.dataLength);
  
  free(msg);
}

void readNewMessage()
{
  MsgHeader header;
  char *msg;
  int err;

  PRINTF("Engine: read new message\n");
  err = readMsgHeader(tmpSocket, &header);
  if(err == 0)
    {
      closesocket(tmpSocket);
      tmpSocket = -1;
      return;
    }
  if(err < 0)
    {
      fprintf(stderr, "Engine: data reading error\n");
      return;
    }

  PRINTF("Reading message length %d\n", header.dataLength );
  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(tmpSocket, msg, header.dataLength);
      if(err == 0)
	{
	  closesocket(tmpSocket);
	  tmpSocket = -1;
	  return;
	}
      if(err < 0)
	{
	  fprintf(stderr, "Engine: data reading error\n");
	  return;
	}
    }
  else
    msg = NULL;
  
  handleNewMessage(&tmpAddr, &header, msg, header.dataLength);
  
  free(msg);
  closesocket(tmpSocket);
  tmpSocket = -1;
}

void readInviteMessage()
{
  MsgHeader header;
  char *msg;
  int err;

  PRINTF("Engine: read invite message\n");
  err = readMsgHeader(inviteSocket, &header);
  if(err == 0)
    {
      /* handle invitor exit */
    }
  if(err < 0)
    {
      fprintf(stderr, "Engine: data reading error\n");
      return;
    }

  PRINTF("Engine: Reading message\n");
  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(inviteSocket, msg, header.dataLength);
      if(err == 0)
	{
	  /* handle invitor exit */
    	}
      if(err < 0)
	{
	  fprintf(stderr, "Engine: data reading error\n");
	  return;
	}
    }
  else
    msg = NULL;
  
  if(header.tag == reqRemoteSessionAddRest)
    remoteClbSessionAddRest(1, &header, msg, header.dataLength);
  else
    fprintf(stderr, "Engine: Unknown remote tag\n");
  
  free(msg);
}



/*********************************************************************
 *
 * Incoming messages handling procedures
 *
 *********************************************************************/
void handleLocalMessage(MsgHeader *header, char *msg, int len)
{
  switch((LocalTag)header->tag)
    {
    case reqLocalSessionCreate:
      PRINTF("Engine: local session create\n");
      localClbSessionCreate(header, msg, len);
      break;

    case reqLocalSessionAddMember:
      PRINTF("Engine: local session add member\n");
      localClbSessionAddMember(header, msg, len);
      break;

    case reqLocalSessionJoin:
      PRINTF("Engine: local session join\n");
      localClbSessionJoin(header, msg, len);
      break;

    case reqLocalSessionReject:
      PRINTF("Engine: local session join\n");
      localClbSessionReject(header, msg, len);
      break;

    case reqLocalSessionGetMembers:
      PRINTF("Engine: local session get mebers\n");
      localClbSessionGetMembers(header, msg, len);
      break;

    case reqLocalSessionQuit:
      PRINTF("Engine: local session quit\n");
      localClbSessionQuit(header, msg, len);
      break;

    case reqLocalSessionSendToApp:
      PRINTF("Engine: local session send to application\n");
      localClbSessionSendToApp(header, msg, len);
      break;

    case reqLocalSessionStartApp:
      PRINTF("Engine: local session start application\n");
      localClbSessionStartApp(header, msg, len);
      break;

    case reqLocalSessionSaveInit:
      PRINTF("Engine: local session save init\n");
      localClbSessionSaveInit(header, msg, len);
      break;

    case reqLocalSessionSaveClose:
      PRINTF("Engine: local session save close\n");
      localClbSessionSaveClose(header, msg, len);
      break;

    case reqLocalSessionRestoreInit:
      PRINTF("Engine: local session restore init\n");
      localClbSessionRestoreInit(header, msg, len);
      break;

    case reqLocalSessionRestoreClose:
      PRINTF("Engine: local session restore close\n");
      localClbSessionRestoreClose(header, msg, len);
      break;

    case reqLocalSessionReset:
      PRINTF("Engine: local session restore close\n");
      localClbSessionReset(header, msg, len);
      break;

    default:
      fprintf(stderr, "Engine: Unknown local tag\n");
    }
  fflush(stdout);
}


void handleAppMessage(int no, MsgHeader *header, char *msg, int len)
{
  switch((AppTag)header->tag)
    {
    case reqAppDettach:
      PRINTF("Engine: local application dettach\n");
      appClbAppDettach(no, header, msg, len);
      break;

    case reqAppData:
      PRINTF("Engine: local application data\n");
      appClbAppData(no, header, msg, len);
      break;

    default:
      fprintf(stderr, "Engine: Unknown application tag\n");
    }
}

void handleNewMessage(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len)
{
  switch((NewTag)header->tag)
    {
    case reqNewRemoteSessionInvite: 
      PRINTF("Engine: remote session invited\n");
      newClbSessionInvite(addr, header, msg, len);
      break;

    case reqNewRemoteSessionConnect: 
      PRINTF("Engine: remote session connect\n");
      newClbSessionConnect(addr, header, msg, len);
      break;

    case reqNewLocalSessionInit:
      PRINTF("Engine: local session init\n");
      newClbSessionInit(addr, header, msg, len);
      break;

    case reqNewLocalApplicationAttach:
      PRINTF("Engine: local application attach\n");
      newClbAppAttach(addr, header, msg, len);
      break;

    default:
      fprintf(stderr, "Engine: Unknown new tag\n");
    }
}

void handleMemberMessage(int no, MsgHeader *header, char *msg, int len)
{
  switch((MemberTag)header->tag)
    {
    case reqRemoteSessionAccept:
      PRINTF("Engine: remote session accept\n");
      remoteClbSessionAccept(no, header, msg, len);
      break;

    case reqRemoteSessionReject:
      PRINTF("Engine: remote: session reject\n");
      remoteClbSessionReject(no, header, msg, len);
      break;

    case reqRemoteSessionAddMember:
      PRINTF("Engine: remote session add member\n");
      remoteClbSessionAddMember(no, header, msg, len);
      break;

    case reqRemoteSessionAddRest:
      PRINTF("Engine: remote session add rest\n");
      remoteClbSessionAddRest(no, header, msg, len);
      break;

    case reqRemoteSessionQuit:
      PRINTF("Engine: remote session quit\n");
      remoteClbSessionQuit(no, header, msg, len);
      break;

    case reqRemoteSessionSendAppData:
      PRINTF("Engine: remote session send applications data\n");
      remoteClbSessionSendAppData(no, header, msg, len);
      break;

    case reqRemoteSessionSendToApp:
      PRINTF("Engine: remote session send to application\n");
      remoteClbSessionSendToApp(no, header, msg, len);
      break;

    case reqRemoteSessionStartApp:
      PRINTF("Engine: remote session start application\n");
      remoteClbSessionStartApp(no, header, msg, len);
      break;

    default:
      fprintf(stderr, "Engine: Unknown member tag %d\n", header->tag);
    }
}



/***********************************************************************
 *
 *                            MAIN
 *
 **********************************************************************/
void main(void)
{
  initDaemon();
  runDaemon();
}










































































