///////////////////////////////////////////////////////////////////////////
// 
// File						: Conference.cpp
//
// Description				: Conference functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-15-96
//
////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "ncs.h"
#include "gsm.h"
#include "TSaudioProc.h"
#include "TSaudioDlg.h"
#include "Conference.h"

#define IDM_EXIT                        32771

CTSaudioDlg *global_dlg;

void Exit()
{
	SendMessage(global_dlg->m_hWnd, WM_CLOSE, 0, NULL); 
}

Conference::Conference(CTSaudioDlg *dlg)
{
	int i;
	HANDLE hProcess;

	main_dlg = dlg;
	global_dlg = dlg;
  
	hProcess = GetCurrentProcess();
	SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);
	
	state = stateStart;
	code_type = ADPCM;
	mode = DUPLEX;
	treshold = 2000;
	
	// ADPCM init
	adpcmsendstate.valprev = 0;
	adpcmsendstate.index = 0;

	for(i=0; i<MAX_USERS; i++)
	{
		adpcmstates[i].valprev = 0;
		adpcmstates[i].index = 0;
		first[i] = 1;
    }

	gsmHandle = gsm_create();
	
	// Start up NCS
	ncsApp = (ncsApplication *)ncsAppAttach(AUDIO_PLUGIN_PORT);

	ncsAppAddCallback(ncsApp, tagAppQuit, (void (*)())Exit);

	if(!hardwareInit())
		SendMessage(main_dlg->m_hWnd, WM_CLOSE, 0, NULL); 

	silence_no = 0;
	current_sender = -1;

	// Create reading thread
	hThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) ReadThread,	// thread function 
			this,									// argument to thread function 
			0,										// use default creation flags 
			&dwThreadId);							// returns the thread identifier 
	// SetThreadPriority(hThread, THREAD_PRIORITY_TIME_CRITICAL);
	
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}

}

int Conference::hardwareInit()
{
	// audio init
	audioInit(this);
	if(!audioWriteInit())
	{
		main_dlg->MessageBox("There is no appropriate audio device", "Info", MB_OK | MB_ICONEXCLAMATION);
		return 0;
	}

	audioWriteClose();

	if(!audioReadInit())
	{
		main_dlg->MessageBox("Audio device does not support audio capturing", "Info", MB_OK | MB_ICONEXCLAMATION);
		audioWriteInit();
	}
	else
		if(!audioWriteInit())
		{
			mode = HALF_DUPLEX;
			main_dlg->MessageBox("Audio device supports only HALF DUPLEX mode", "Info", MB_OK | MB_ICONEXCLAMATION);
		}

	return 1;
}


DWORD Conference::ReadThread(LPDWORD lpdwParam)
{
	struct timeval timeout;
	fd_set set;
	ncsData *ncsdata;
	int size;
	int sockfd;
	char *data;
	ncsApplication *app = ((Conference *)lpdwParam)->ncsApp;

	timeout.tv_sec = 0;
	timeout.tv_usec = 1000;

	while(1)
	{
		sockfd = ncsAppGetFD(app, &set);
		if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
		{
			ncsAppProcessIncoming(app, &set);
			size = ncsAppGetData(app, &set, &data);
			if(size > 0)
			{
				size = size - sizeof(ncsData) + 1;
				ncsdata = (ncsData *)data;
				ncsdata->handle = ntohl(ncsdata->handle);
				ncsdata->type = ntohl(ncsdata->type);
				((Conference *)lpdwParam)->HandleData(ncsdata->type, ncsdata->data, size, 
								       ncsdata->handle);
				free(data);
			}
		}
	}
	return 1;
}

void Conference::HandleData(DWORD tag, void *data, int len, DWORD handle)
{
	char buffer[1600];
	short level;
	int i;
	
	// MessageBox(NULL, "Get audio data", "Info", MB_OK);
	switch (tag)
	{
	case ADPCM:
		//if((mode == DUPLEX) || (state == stateStop))
		//{
			adpcm_decoder((char *)data, (short *)buffer, 800, &adpcmstates[handle]);
		//}
		break;

    case GSM:
		for(i=0; i<5; i++)
		{
			gsm_decode(gsmHandle, ((gsm_frame *)data)[i], (short *)&buffer[i * 320]);
		}

		break;
	}
	
	level = audioGetLevel(buffer);
	main_dlg->m_LedOut.SetPos(level/330);
	//level = 0;
	
	if(mode == HALF_DUPLEX)
	{
		if(state == stateStart)
		{
			if(level > treshold)
			{
				StopSending();
				current_sender = handle;
				audioWriteChunk(buffer);
			}
		}
		else
		{
			if(handle == current_sender)
			{
				audioWriteChunk(buffer);
				if(level <= treshold)
				{
					silence_no++;
					if(silence_no == 10)
					{
						StartSending();
						silence_no = 0;
					}
				}
			}
		}
	}
	else
	{
		if(current_sender < 0)
		{
			if(level > treshold)
			{
				current_sender = handle;
				audioWriteChunk(buffer);
			}
		}
		else
		{
			if(handle == current_sender)
			{
				audioWriteChunk(buffer);
				if(level <= treshold)
				{
					silence_no++;
					if(silence_no == 10)
					{
						current_sender = -1;
						silence_no = 0;
					}
				}
			}			
		}
	}
}

void Conference::SendData(char *data)
{
   audioPluginMessageStructure *msg;
   gsm_frame *gsm_msg;
   int i;
   short level;

	level = audioGetLevel(data);
   switch(code_type)
	{
	case GSM:
		gsm_msg = (gsm_frame *)malloc(5 * sizeof(gsm_frame));
		for(i=0; i<5; i++)
		{
			gsm_encode(gsmHandle, (gsm_signal *)(&data[320 * i]), gsm_msg[i]);
		}
		ncsAppSendData(ncsApp, -1, GSM, gsm_msg, 5*sizeof(gsm_frame));
		free(gsm_msg);
	    break;
	      
	case ADPCM:
		msg = (audioPluginMessageStructure *)malloc(sizeof(audioPluginMessageStructure));
		if (!msg)
		{
			fprintf(stderr, "Message allocation error\n");
			return;
		}
		// MessageBox(NULL, "Sending audio data", "Info", MB_OK);
		adpcm_coder((short *)data, msg->buf, 800, &adpcmsendstate);
		ncsAppSendData(ncsApp, -1, ADPCM, msg, sizeof(audioPluginMessageStructure));
		free(msg);
		break;
	}
   	
   main_dlg->m_LedIn.SetPos(level/330);

}


void Conference::StartSending(void)
{
	if(state == stateStop)
	{
		state = stateStart;
		if(mode == HALF_DUPLEX)
		{
			main_dlg->m_LedIn.SetPos(0);
			audioWriteClose();
			audioReadInit();
		}
		audioReadStart();
	}
}

void Conference::StopSending(void)
{
	if(state == stateStart)
	{
		if(mode == HALF_DUPLEX)
		{
			audioReadClose();
			main_dlg->m_LedOut.SetPos(0);
			audioWriteInit();
		}
		else
			audioReadStop();
		state = stateStop;
	}
}

void Conference::setTreshold(UINT level)
{
	treshold = level * 327;
}

void Conference::codeADPCM(void)
{
	code_type = ADPCM;
}

void Conference::codeGSM(void)
{
	code_type = GSM;
}

Conference::~Conference(void)
{
	TerminateThread(hThread, 0);
	CloseHandle(hThread);

	if(mode == HALF_DUPLEX)
	{
		if(state == stateStart)
			audioReadClose();
		else
			audioWriteClose();
	}
	else
	{
		audioReadClose();
		audioWriteClose();
	}

	gsm_destroy(gsmHandle);
	audioClose();
	ncsAppDettach(ncsApp);
}
