///////////////////////////////////////////////////////////////////////////
// 
// File						: capture.cpp
//
// Description				: capture functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-19-96
//
////////////////////////////////////////////////////////////////////////////
#include <vfw.h>

#include "stdafx.h"
#include "DisplayDlg.h"
#include "h261.h"
#include "h263.h"
#include "capture.h"
#include "conference.h"
#include "VideoProtocol.h"
#include "convert.h"
#include "display.h"

#define RGB24 1
#define YUV9 2

DisplayDlg *display;		// display dialog
HWND capWnd;				// capture window handle
HDRAWDIB hDraw;				// drawing DIB handle
BITMAPINFO *info;			// video format info
HDC hDc;					// drawing context handle

int frames11=0;				// number of frames captures
int totalsize = 0;			// total size of all frames captured
int first = 1;				// indicator of first frame
int format = RGB24;

////////////////////////////////////////////////////////////////////////
//
// Capture initializtion procedure
//
int captureInit(HWND parent, int msec)
{
	CAPTUREPARMS params;
	
	// mirror display dialog
	//display = new DisplayDlg;

	//if(!display->Create(IDD_DISPLAY))
	//	MessageBox(NULL, "Error creating window", "Error", MB_OK);

	//display->SetWindowPos(&display->wndTopMost, 100, 280, 176, 170, 0);
	display = displayOpen(MAX_USERS);

	// capture window
	capWnd = capCreateCaptureWindow(
		(LPSTR) "Mirror",
		WS_CHILD | WS_VISIBLE,
		0, 0, 176, 144,
		display->m_hWnd,
		23);

	if(capWnd == NULL)
		return 0;

	// connect to capture driver
	capDriverConnect(capWnd, 0);

	// set video format
	captureSetFormat();
	
	// initialize drawing procedures
	//drawOpen();

	// set callback for frame
	capSetCallbackOnVideoStream(capWnd, captureFrameClb);

	// set setup
	capCaptureGetSetup(capWnd, &params, sizeof(CAPTUREPARMS));
    params.dwRequestMicroSecPerFrame = msec * 1000; 
    params.fMakeUserHitOKToCapture = FALSE; 
    params.wPercentDropForError = 10; 
    params.fYield = TRUE;
    params.wNumVideoRequested = 20; 
    params.fCaptureAudio = FALSE; 
    params.wNumAudioRequested = 1; 
    params.fLimitEnabled = FALSE; 
    params.wTimeLimit = 30; 
	params.fAbortLeftMouse = FALSE; 
    params.fAbortRightMouse = FALSE; 

	capCaptureSetSetup(capWnd, &params, sizeof(CAPTUREPARMS));

	// set overlay preview mode
	//capOverlay(capWnd, TRUE);
	
	// set preview mode
	//capPreviewScale(capWnd, FALSE);
	//capPreviewRate(capWnd, 100);
	capPreview(capWnd, FALSE);

	return 1;
}

////////////////////////////////////////////////////////////////////////
//
// Capture closing procedure
//
void captureClose(void)
{
	char text[100];

	//wsprintf(text, "Bytes/frame: %d", totalsize/frames11);
	//MessageBox(NULL, text, "Info", MB_OK);

	//display->DestroyWindow();
	//delete display;
	capSetCallbackOnFrame(capWnd, NULL);
	capDriverDisconnect(capWnd);
	DestroyWindow(capWnd);
	displayClose(MAX_USERS);
}

////////////////////////////////////////////////////////////////////////
//
// Displaying video format info
//
void captureDisplayInfo(void)
{
	char text[100];

	wsprintf(text, "Bit count: %d", info->bmiHeader.biBitCount);
	MessageBox(NULL, text, "Info", MB_OK);
	wsprintf(text, "Width: %d", info->bmiHeader.biWidth);
	MessageBox(NULL, text, "Info", MB_OK);
	wsprintf(text, "Height: %d", info->bmiHeader.biHeight);
	MessageBox(NULL, text, "Info", MB_OK);
	wsprintf(text, "Compression: %d", info->bmiHeader.biCompression);
	MessageBox(NULL, text, "Info", MB_OK);
	wsprintf(text, "Size: %d", info->bmiHeader.biSizeImage);
	MessageBox(NULL, text, "Info", MB_OK);
}

////////////////////////////////////////////////////////////////////////
//
// Capture setting format procedure
//
int captureSetFormat(void)
{
	DWORD info_size;
	char text[100];

	// getting current format settings
	info_size = capGetVideoFormatSize(capWnd);
	info = (BITMAPINFO *)GlobalAlloc(GPTR, info_size);
	capGetVideoFormat(capWnd, info, info_size);

	// BITMAPINFO structure initialization (video format)
	info->bmiHeader.biWidth = 240;
	info->bmiHeader.biHeight = 180;
	info->bmiHeader.biPlanes = 1;
	info->bmiHeader.biBitCount = 24;
	info->bmiHeader.biCompression = BI_RGB;
	
	// new format setting
	if(!capSetVideoFormat(capWnd, info, sizeof(BITMAPINFO)))
	{
		info->bmiHeader.biWidth = 240;
		info->bmiHeader.biHeight = 180;
		// info->bmiHeader.biPlanes = 1;
		//info->bmiHeader.biBitCount = 24;
		info->bmiHeader.biCompression = BI_YUV9; //BI_RGB;

		format = YUV9;

		if(!capSetVideoFormat(capWnd, info, sizeof(BITMAPINFO)))
		{
			MessageBox(NULL, "Can't set video format", "Error", MB_OK);
			return 0;
		}
	}

	return 1;
}

////////////////////////////////////////////////////////////////////////
//
// Capture staring procedure
//
void captureGrab(void)
{
		
	if(!capCaptureSequenceNoFile(capWnd))
		MessageBox(NULL, "Capture error", "Error", MB_OK);
		
}

////////////////////////////////////////////////////////////////////////
//
// Capture frame callback
//
LRESULT PASCAL captureFrameClb(HWND hWnd, LPVIDEOHDR lpVHdr)
{
	char text[100];

	//wsprintf(text, "Data lenght: %d", lpVHdr->dwBufferLength);
	//MessageBox(NULL, text, "Info", MB_OK);
	captureHandleImage(lpVHdr->lpData);
	return TRUE;
}
 
////////////////////////////////////////////////////////////////////////
//
// Capture frame handler (compressing + sending to members)
//
void captureHandleImage(void *image)
{
	unsigned char *tmp;

	if(format == RGB24)
	{		
		tmp = (unsigned char *)malloc(3*176*144/2);
		//RGB24toQCIF((char *)image, 240, 180, tmp);
		RGB24toYUV((unsigned char *)image, tmp, &tmp[25344], &tmp[25344+6336], 240, 180);
		confCodeImage((char *)tmp, 0);
		free(tmp);
	}
	else
	{
		confCodeImage((char *)image, 1);
	}
}

////////////////////////////////////////////////////////////////////////
//
// Capture rate setting procedure
//
void captureSetRate(int msec)
{
	CAPTUREPARMS params;
	char text[100];


	capDriverDisconnect(capWnd);
	DestroyWindow(capWnd);
	capWnd = capCreateCaptureWindow(
		(LPSTR) "Mirror",
		WS_CHILD | WS_VISIBLE,
		0, 0, 176, 144,
		display->m_hWnd,
		23);

	// connect to capture driver
	capDriverConnect(capWnd, 0);

	captureSetFormat();

	// set callback for frame
	capSetCallbackOnVideoStream(capWnd, captureFrameClb);

	//capCaptureStop(capWnd);

	capCaptureGetSetup(capWnd, &params, sizeof(CAPTUREPARMS));

	//wsprintf(text, "Get %d, Set %d",params.dwRequestMicroSecPerFrame/1000, msec);
	//MessageBox(NULL, text, "Info", MB_OK);

	if(msec >= 85)
		params.dwRequestMicroSecPerFrame = msec * 1000; 
	else
		params.dwRequestMicroSecPerFrame = 85 * 1000; 

    params.fMakeUserHitOKToCapture = FALSE; 
    params.wPercentDropForError = 10; 
    params.fYield = TRUE;
    params.wNumVideoRequested = 20; 
    params.fCaptureAudio = FALSE; 
    params.wNumAudioRequested = 1; 
    params.fLimitEnabled = FALSE; 
    params.wTimeLimit = 30; 
	params.fAbortLeftMouse = FALSE; 
    params.fAbortRightMouse = FALSE; 

	if(!capCaptureSetSetup(capWnd, &params, sizeof(CAPTUREPARMS)))
		MessageBox(NULL, "Capture set setup error", "Error", MB_OK);

	captureGrab();
}

////////////////////////////////////////////////////////////////////////
//
// Drawing procedures
//
void drawOpen(void)
{
	CClientDC *cdc;
	PAINTSTRUCT paintst;
	
	cdc = new CClientDC(display);
	cdc->SetMapMode(MM_TEXT);
	hDc = cdc->GetSafeHdc();

	hDraw = DrawDibOpen();
	// info->bmiHeader.biBitCount = 32;
	//if(!DrawDibProfileDisplay(&info->bmiHeader))
	//	MessageBox(NULL, "Error creating draw dib", "Error", MB_OK);
	
	if(!DrawDibBegin(hDraw, hDc, 160, 120, &info->bmiHeader, -1, -1, DDF_HALFTONE ))
		MessageBox(NULL, "Draw Dib begin error", "Error", MB_OK);
}

void draw(void *image)
{
	if(!DrawDibDraw(hDraw, hDc, 0, 0, 160, 120, &info->bmiHeader, image, 0, 0, 160, 120, DDF_SAME_DRAW))// DDF_HALFTONE
		MessageBox(NULL, "Draw Dib draw error", "Error", MB_OK);
}

